#!/usr/bin/python

import subprocess

__all__ = ['Colors']


class ANSIColors(object):
    ESCAPE = '\033['
    TRAIL = 'm'
    PREFIX = ''

    def __init__(self):
        self.COLORS = self.get_dict()
        self.ORDER = self.get_order()

    @classmethod
    def get_color(cls, color):
        code = cls.COLORS.get(color, None)
        if None:
            return ''
        return '{0}{1}{2}'.format(cls.ESCAPE, code, cls.TRAIL)

    @classmethod
    def get_all(cls):
        return sorted([cls.get_color(key) for key in cls.COLORS])

    @classmethod
    def get_dict(cls):
        colors = {}
        for key in cls.COLORS:
            color = cls.get_color(key)
            colors['{0}{1}'.format(cls.PREFIX, key)] = color
        return colors

    @classmethod
    def get_order(cls):
        code_list = sorted([(code, key) for key, code in cls.COLORS.items()])
        return [key for code, key in code_list]


class FgColors(ANSIColors):
    COLORS = {
        'black': '30',
        'red': '31',
        'green': '32',
        'yellow': '33',
        'blue': '34',
        'magenta': '35',
        'cyan': '36',
        'white': '37',
        'reset': '39',
    }


class BgColors(ANSIColors):
    PREFIX = 'bg_'
    COLORS = {
        'black': '40',
        'red': '41',
        'green': '42',
        'yellow': '43',
        'blue': '44',
        'magenta': '45',
        'cyan': '46',
        'white': '47',
        'reset': '49',
    }


class StyleColors(ANSIColors):
    COLORS = {
        'reset_all': '0',
        'bright': '1',
        'dim': '2',
        'normal': '22',
    }


class Colors(object):

    fg = FgColors()
    bg = BgColors()
    st = StyleColors()
    formats = ['bold', 'underline']
    styles = ['dim', 'normal', 'bright']

    def __init__(self):
        """ Checks if the shell supports colors """

        try:
            p = subprocess.Popen(['tput', 'colors'],
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE)
            num_colors = int(p.stdout.read())
        except (OSError, ValueError):
            num_colors = 1

        self.has_colors = False
        if num_colors > 1:
            self.has_colors = True

        self.enable_colors()

        self.COLORS = self.enumerate_colors()

    def enumerate_colors(self):
        colors = {}
        cls_list = [self.fg, self.bg, self.st]
        for cls in cls_list:
            colors.update(cls.get_dict())
        return colors

    def enable_colors(self):
        """ Method to enable colors """
        self.colors_enabled = True

    def disable_colors(self):
        """ Method to disable colors """
        self.colors_enabled = False

    def _wrap_color(self, code, text, format=None, style=None):
        """ Colors text with code and given format """
        color = None
        if code[:3] == self.bg.PREFIX:
            color = self.bg.COLORS.get(code, None)
        if not color:
            color = self.fg.COLORS.get(code, None)

        if not color:
            raise Exception('Color code not found')

        if format and format not in self.formats:
            raise Exception('Color format not found')

        fmt = "0;"
        if format == 'bold':
            fmt = "1;"
        elif format == 'underline':
            fmt = "4;"

        # Manage the format
        parts = color.split('[')
        color = '{0}[{1}{2}'.format(parts[0], fmt, parts[1])

        if self.has_colors and self.colors_enabled:
            # Set brightness
            st = ''
            if style:
                st = self.st.COLORS.get(style, '')
            return "{0}{1}{2}{3}".format(st, color, text, self.st.COLORS['reset_all'])
        else:
            return text

    # Foreground
    def black(self, text, format=None, style=None):
        return self._wrap_color('black', text, format=format, style=style)

    def red(self, text, format=None, style=None):
        return self._wrap_color('red', text, format=format, style=style)

    def green(self, text, format=None, style=None):
        return self._wrap_color('green', text, format=format, style=style)

    def yellow(self, text, format=None, style=None):
        return self._wrap_color('yellow', text, format=format, style=style)

    def blue(self, text, format=None, style=None):
        return self._wrap_color('blue', text, format=format, style=style)

    def magenta(self, text, format=None, style=None):
        return self._wrap_color('magenta', text, format=format, style=style)

    def cyan(self, text, format=None, style=None):
        return self._wrap_color('cyan', text, format=format, style=style)

    def white(self, text, format=None, style=None):
        return self._wrap_color('white', text, format=format, style=style)

    def reset(self, text, format=None, style=None):
        return self._wrap_color('reset', text, format=format, style=style)

    # Background
    def bg_black(self, text, format=None, style=None):
        return self._wrap_color('bg_black', text, format=format, style=style)

    def bg_red(self, text, format=None, style=None):
        return self._wrap_color('bg_red', text, format=format, style=style)

    def bg_green(self, text, format=None, style=None):
        return self._wrap_color('bg_green', text, format=format, style=style)

    def bg_yellow(self, text, format=None, style=None):
        return self._wrap_color('bg_yellow', text, format=format, style=style)

    def bg_blue(self, text, format=None, style=None):
        return self._wrap_color('bg_blue', text, format=format, style=style)

    def bg_magenta(self, text, format=None, style=None):
        return self._wrap_color('bg_magenta', text, format=format, style=style)

    def bg_cyan(self, text, format=None, style=None):
        return self._wrap_color('bg_cyan', text, format=format, style=style)

    def bg_white(self, text, format=None, style=None):
        return self._wrap_color('bg_white', text, format=format, style=style)

    def bg_reset(self, text, format=None, style=None):
        return self._wrap_color('bg_reset', text, format=format, style=style)



#!/usr/bin/python

import colors

def main():
    c = colors.Colors()

    COLUMN = 7
    for bg_color in c.bg.ORDER:
        bg_fn = getattr(c, '{0}{1}'.format(c.bg.PREFIX, bg_color))
        for fmt in [''] + c.formats:
            line = ''
            for fg_color in c.fg.ORDER:
                if fg_color != 'reset':
                    fg = fg_color[:3]
                    fg_fn = getattr(c, fg_color)
                    for st in c.styles:
                        colored_text = fg_fn(fg, format=fmt, style=st)
                        # Trick to format correctly
                        line += ' ' * (COLUMN - len(fg)) + colored_text
            line += c.st.COLORS['reset_all']

            print(bg_fn(line))
        print('\n')


if __name__ == "__main__":
    main()



#!/usr/bin/python

import sys
import unittest

# Third party apps
import mock

# This app
import colors

# One test doesn't work right in python 2.6
py26 = False
if sys.version_info < (2, 7):
    py26 = True


class TestColors(unittest.TestCase):

    def setUp(self):
        self.patcher = mock.patch('subprocess.Popen')
        self.subprocess = self.patcher.start()
        self.p = mock.Mock()
        self.p.stdout.read.return_value = '8'
        self.subprocess.return_value = self.p
        self.colors = colors.Colors()
        self.assertTrue(self.colors.colors_enabled)

    def tearDown(self):
        self.patcher.stop()

    def test_color_text(self):
        output = self.colors.red('This will be red text')
        expected = '\x1b[0;31mThis will be red text\x1b[0m'
        self.assertEqual(output, expected)

    def test_two_colors_text(self):
        output = self.colors.red('This will be red text') + \
            self.colors.green('and this will be green text.')
        expected = '\x1b[0;31mThis will be red text\x1b[0m\x1b[0;32m' \
                   'and this will be green text.\x1b[0m'
        self.assertEqual(output, expected)

    def test_colors_from_dictionary(self):
        output = '{red}This will be red text {green}' \
                 'and this will be green text.{reset_all}'.format(**self.colors.COLORS)
        expected = '\x1b[31mThis will be red text \x1b[32m' \
                   'and this will be green text.\x1b[0m'
        self.assertEqual(output, expected)

    def test_colors_format_bold(self):
        output = self.colors.red('This will be BOLD red text',
                            format='bold')
        expected = '\x1b[1;31mThis will be BOLD red text\x1b[0m'
        self.assertEqual(output, expected)

    def test_colors_format_underline(self):
        output = self.colors.red('This will be UNDERLINE red text',
                            format='underline')
        expected = '\x1b[4;31mThis will be UNDERLINE red text\x1b[0m'
        self.assertEqual(output, expected)

    def test_colors_format_background(self):
        output = self.colors.bg_red('This will be BACKGROUND red text')
        expected = '\x1b[0;41mThis will be BACKGROUND red text\x1b[0m'
        self.assertEqual(output, expected)

    def test_disable_colors(self):
        self.colors.enable_colors()
        self.assertTrue(self.colors.colors_enabled)
        output = self.colors.red('This will be red text')
        expected = '\x1b[0;31mThis will be red text\x1b[0m'
        self.assertEqual(output, expected)
        self.colors.disable_colors()
        self.assertFalse(self.colors.colors_enabled)
        output = self.colors.red('This will be red text')
        expected = 'This will be red text'
        self.assertEqual(output, expected)

    def test_enable_colors(self):
        self.colors.disable_colors()
        self.assertFalse(self.colors.colors_enabled)
        output = self.colors.red('This will be red text')
        expected = 'This will be red text'
        self.assertEqual(output, expected)
        self.colors.enable_colors()
        self.assertTrue(self.colors.colors_enabled)
        output = self.colors.red('This will be red text')
        expected = '\x1b[0;31mThis will be red text\x1b[0m'
        self.assertEqual(output, expected)

    def test_wrap_colors_bad_code(self):
        if py26:
            self.assertRaises(Exception, self.colors._wrap_color, 'bad', 'This should fail')
        else:
            with self.assertRaises(Exception) as cm:
                self.colors._wrap_color('bad', 'This should fail')
            e = cm.exception
            self.assertEqual(str(e), 'Color code not found')

    def test_wrap_colors_bad_format(self):
        if py26:
            self.assertRaises(Exception, self.colors._wrap_color, 'red', 'This should fail', format='bad')
        else:
            with self.assertRaises(Exception) as cm:
                self.colors._wrap_color('red', 'This should fail', format='bad')
            e = cm.exception
            self.assertEqual(str(e), 'Color format not found')


class TestColorsUnavailable(unittest.TestCase):

    def setUp(self):
        self.patcher = mock.patch('subprocess.Popen')
        self.subprocess = self.patcher.start()

    def tearDown(self):
        self.patcher.stop()

    def test_colors_not_available(self):
        self.p = mock.Mock()
        self.p.stdout.read.return_value = '1'
        self.subprocess.return_value = self.p
        c = colors.Colors()
        self.assertFalse(c.has_colors)
        output = c.red('This will be red text')
        expected = 'This will be red text'
        self.assertEqual(output, expected)

    def test_colors_not_available_oserror(self):
        self.p = mock.Mock()
        self.p.stdout.read.side_effect = OSError
        self.subprocess.return_value = self.p
        c = colors.Colors()
        self.assertFalse(c.has_colors)
        output = c.red('This will be red text')
        expected = 'This will be red text'
        self.assertEqual(output, expected)

    def test_colors_not_available_valueerror(self):
        self.p = mock.Mock()
        self.p.stdout.read.side_effect = ValueError
        self.subprocess.return_value = self.p
        c = colors.Colors()
        self.assertFalse(c.has_colors)
        output = c.red('This will be red text')
        expected = 'This will be red text'
        self.assertEqual(output, expected)


def run_tests(test_names=None, test_class_list=None, module=None, verbose=2):
    suite = unittest.TestSuite()
    loader = unittest.TestLoader()
    if test_names:
        tests = loader.loadTestsFromNames(test_names, module=module)
        suite.addTests(tests)
    else:
        if not test_class_list:
            return
        for test_class in test_class_list:
            tests = loader.loadTestsFromTestCase(test_class)
            suite.addTests(tests)
    unittest.TextTestRunner(verbosity=verbose).run(suite)


if __name__ == '__main__':
    test_class_list = [TestColors, TestColorsUnavailable]
    run_tests(test_names=sys.argv[1:],
              test_class_list=test_class_list,
              module=sys.modules[__name__])



from distutils.core import setup

setup(
    name = 'pycolors2',
    py_modules = ['colors',],
    version = '0.0.4',
    author = 'Chris Gilmer',
    author_email = 'chris.gilmer@gmail.com',
    maintainer = 'Chris Gilmer',
    maintainer_email = 'chris.gilmer@gmail.com',

    url = 'http://github.com/chrisgilmerproj/pycolors2',

    license = 'MIT license',
    description = """ Tool to color code python output """,

    long_description = open('README.markdown').read(),
    requires = [],

    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System',
        'Topic :: Terminals',
        'Topic :: Utilities',
    ],
)
